package java.awt.image;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import sun.awt.image.ImagingLib;
import java.awt.RenderingHints;

public class LookupOp implements BufferedImageOp, RasterOp
{
    private LookupTable ltable;
    private int numComponents;
    RenderingHints hints;
    
    public LookupOp(LookupTable ltable, RenderingHints hints) {
        this.ltable = ltable;
        this.hints = hints;
        this.numComponents = this.ltable.getNumComponents();
    }
    
    public final LookupTable getTable() {
        return this.ltable;
    }
    
    @Override
    public final BufferedImage filter(BufferedImage bufferedImage, BufferedImage dst) {
        ColorModel colorModel = bufferedImage.getColorModel();
        int numColorComponents = colorModel.getNumColorComponents();
        if ((colorModel instanceof IndexColorModel)) {
            throw new IllegalArgumentException("LookupOp cannot be performed on an indexed image");
        }
        int numComponents = this.ltable.getNumComponents();
        if (numComponents != 1 && numComponents != colorModel.getNumComponents() && numComponents != colorModel.getNumColorComponents()) {
            throw new IllegalArgumentException("Number of arrays in the  lookup table (" + numComponents + " is not compatible with the  src image: " + bufferedImage);
        }
        boolean n = false;
        int width = bufferedImage.getWidth();
        int height = bufferedImage.getHeight();
        if (dst == null) {
            dst = this.createCompatibleDestImage(bufferedImage, null);
        }
        else {
            if (width != dst.getWidth()) {
                throw new IllegalArgumentException("Src width (" + width + ") not equal to dst width (" + dst.getWidth() + ")");
            }
            if (height != dst.getHeight()) {
                throw new IllegalArgumentException("Src height (" + height + ") not equal to dst height (" + dst.getHeight() + ")");
            }
            colorModel = dst.getColorModel();
            if (colorModel.getColorSpace().getType() != colorModel.getColorSpace().getType()) {
                n = true;
                dst = this.createCompatibleDestImage(bufferedImage, null);
            }
        }
        if (ImagingLib.filter(this, bufferedImage, dst) == null) {
            WritableRaster writableRaster = bufferedImage.getRaster();
            WritableRaster writableRaster2 = dst.getRaster();
            if (colorModel.hasAlpha() && (numColorComponents - 1 == numComponents || numComponents == 1)) {
                int minX = writableRaster.getMinX();
                int minY = writableRaster.getMinY();
                int[] array = new int[numColorComponents - 1];
                for (int i = 0; i < numColorComponents - 1; ++i) {
                    array[i] = i;
                }
                writableRaster = writableRaster.createWritableChild(minX, minY, writableRaster.getWidth(), writableRaster.getHeight(), minX, minY, array);
            }
            if (colorModel.hasAlpha() && (writableRaster2.getNumBands() - 1 == numComponents || numComponents == 1)) {
                int minX2 = writableRaster2.getMinX();
                int minY2 = writableRaster2.getMinY();
                int[] array2 = new int[numColorComponents - 1];
                for (int j = 0; j < numColorComponents - 1; ++j) {
                    array2[j] = j;
                }
                writableRaster2 = writableRaster2.createWritableChild(minX2, minY2, writableRaster2.getWidth(), writableRaster2.getHeight(), minX2, minY2, array2);
            }
            this.filter(writableRaster, writableRaster2);
        }
        if (!(n == 0)) {
            new ColorConvertOp(this.hints).filter(dst, dst);
            return dst;
        }
        return dst;
    }
    
    @Override
    public final WritableRaster filter(Raster raster, WritableRaster compatibleDestRaster) {
        int numBands = raster.getNumBands();
        compatibleDestRaster.getNumBands();
        int height = raster.getHeight();
        int width = raster.getWidth();
        int[] array = new int[numBands];
        if (compatibleDestRaster == null) {
            compatibleDestRaster = this.createCompatibleDestRaster(raster);
        }
        else if (height != compatibleDestRaster.getHeight() || width != compatibleDestRaster.getWidth()) {
            throw new IllegalArgumentException("Width or height of Rasters do not match");
        }
        int numBands2 = compatibleDestRaster.getNumBands();
        if (numBands != numBands2) {
            throw new IllegalArgumentException("Number of channels in the src (" + numBands + ") does not match number of channels in the destination (" + numBands2 + ")");
        }
        int numComponents = this.ltable.getNumComponents();
        if (numComponents != 1 && numComponents != raster.getNumBands()) {
            throw new IllegalArgumentException("Number of arrays in the  lookup table (" + numComponents + " is not compatible with the  src Raster: " + raster);
        }
        if (ImagingLib.filter(this, raster, compatibleDestRaster) != null) {
            return compatibleDestRaster;
        }
        if ((this.ltable instanceof ByteLookupTable)) {
            this.byteFilter((ByteLookupTable)this.ltable, raster, compatibleDestRaster, width, height, numBands);
        }
        else if ((this.ltable instanceof ShortLookupTable)) {
            this.shortFilter((ShortLookupTable)this.ltable, raster, compatibleDestRaster, width, height, numBands);
        }
        else {
            int minX = raster.getMinX();
            int minY = raster.getMinY();
            int minX2 = compatibleDestRaster.getMinX();
            for (int minY2 = compatibleDestRaster.getMinY(), i = 0; i < height; ++i, ++minY, ++minY2) {
                for (int j = 0; j < width; ++j, ++minX, ++minX2) {
                    raster.getPixel(minX, minY, array);
                    this.ltable.lookupPixel(array, array);
                    compatibleDestRaster.setPixel(minX2, minY2, array);
                }
            }
        }
        return compatibleDestRaster;
    }
    
    @Override
    public final Rectangle2D getBounds2D(BufferedImage bufferedImage) {
        return this.getBounds2D(bufferedImage.getRaster());
    }
    
    @Override
    public final Rectangle2D getBounds2D(Raster raster) {
        return raster.getBounds();
    }
    
    @Override
    public BufferedImage createCompatibleDestImage(BufferedImage bufferedImage, ColorModel colorModel) {
        int width = bufferedImage.getWidth();
        int height = bufferedImage.getHeight();
        boolean n = false;
        if (colorModel == null) {
            ColorModel colorModel2 = bufferedImage.getColorModel();
            WritableRaster raster = bufferedImage.getRaster();
            if ((colorModel2 instanceof ComponentColorModel)) {
                DataBuffer dataBuffer = raster.getDataBuffer();
                int hasAlpha = colorModel2.hasAlpha();
                int alphaPremultiplied = colorModel2.isAlphaPremultiplied();
                int transparency = colorModel2.getTransparency();
                int[] array = null;
                if ((this.ltable instanceof ByteLookupTable)) {
                    if (dataBuffer.getDataType() == 1) {
                        if (!(hasAlpha == 0)) {
                            array = new int[2];
                            if (transparency == 2) {
                                array[1] = 1;
                            }
                            else {
                                array[1] = 8;
                            }
                        }
                        else {
                            array = new int[] { 0 };
                        }
                        array[0] = 8;
                    }
                }
                else if ((this.ltable instanceof ShortLookupTable)) {
                    n = true;
                    if (dataBuffer.getDataType() == 0) {
                        if (!(hasAlpha == 0)) {
                            array = new int[2];
                            if (transparency == 2) {
                                array[1] = 1;
                            }
                            else {
                                array[1] = 16;
                            }
                        }
                        else {
                            array = new int[] { 0 };
                        }
                        array[0] = 16;
                    }
                }
                if (array != null) {
                    colorModel2 = new ComponentColorModel(colorModel2.getColorSpace(), array, (boolean)hasAlpha, (boolean)alphaPremultiplied, transparency, n);
                }
            }
            return new BufferedImage(colorModel2, colorModel2.createCompatibleWritableRaster(width, height), colorModel2.isAlphaPremultiplied(), null);
        }
        else {
            return new BufferedImage(colorModel, colorModel.createCompatibleWritableRaster(width, height), colorModel.isAlphaPremultiplied(), null);
        }
    }
    
    @Override
    public WritableRaster createCompatibleDestRaster(Raster raster) {
        return raster.createCompatibleWritableRaster();
    }
    
    @Override
    public final Point2D getPoint2D(Point2D point2D, Point2D point2D2) {
        if (point2D2 == null) {
            point2D2 = new Point2D.Float();
        }
        point2D2.setLocation(point2D.getX(), point2D.getY());
        return point2D2;
    }
    
    @Override
    public final RenderingHints getRenderingHints() {
        return this.hints;
    }
    
    private final void byteFilter(ByteLookupTable byteLookupTable, Raster raster, WritableRaster writableRaster, int n, int n2, int n3) {
        int[] samples = null;
        byte[][] table = byteLookupTable.getTable();
        int offset = byteLookupTable.getOffset();
        int n4 = 1;
        if (table.length == 1) {
            n4 = 0;
        }
        int length = table[0].length;
        for (int i = 0; i < n2; ++i) {
            for (int n5 = 0, j = 0; j < n3; ++j, n5 += n4) {
                samples = raster.getSamples(0, i, n, 1, j, samples);
                int k = 0;
                while (k < n) {
                    int l = samples[k] - offset;
                    if (l < 0 || l > length) {
                        throw new IllegalArgumentException("index (" + l + "(out of range:  srcPix[" + k + "]=" + samples[k] + " offset=" + offset);
                    }
                    samples[k] = table[n5][l];
                    ++k;
                }
                writableRaster.setSamples(0, i, n, 1, j, samples);
            }
        }
    }
    
    private final void shortFilter(ShortLookupTable shortLookupTable, Raster raster, WritableRaster writableRaster, int n, int n2, int n3) {
        int[] samples = null;
        short[][] table = shortLookupTable.getTable();
        int offset = shortLookupTable.getOffset();
        int n4 = 1;
        if (table.length == 1) {
            n4 = 0;
        }
        int n5 = 65535;
        for (int i = 0; i < n2; ++i) {
            for (int n6 = 0, j = 0; j < n3; ++j, n6 += n4) {
                samples = raster.getSamples(0, i, n, 1, j, samples);
                int k = 0;
                while (k < n) {
                    int l = samples[k] - offset;
                    if (l < 0 || l > n5) {
                        throw new IllegalArgumentException("index out of range " + l + " x is " + k + "srcPix[x]=" + samples[k] + " offset=" + offset);
                    }
                    samples[k] = table[n6][l];
                    ++k;
                }
                writableRaster.setSamples(0, i, n, 1, j, samples);
            }
        }
    }
}
